/*
"Boom Boom Driller

  Copyright (C) 2010 Jari Komppa

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

http://iki.fi/sol/
*/

#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <windows.h>
#include "SDL.h"
#include "SDL_image.h"
#include "cfl.h"
#include "tinyxml.h"
#include "MersenneTwister.h"
#include "ramfile3.h"

#define TITLE "Boom! Boom! Driller! v1.02"

#define SMOOTHSTEP(x) ((x)*(x)*(3-2*(x)))

#define UNUSED_PARTICLE 0xc0cac01a // in the 'hits' bit

// Screen pitch
#define PITCH (gScreen->pitch / 4)
// Screen width
#define WIDTH 800
// Screen height
#define HEIGHT 600
// Definition of PI
#define PI 3.1415926535897932384626433832795f

// Physics iterations per second
#define PHYSICSFPS 50

enum GRAINTYPES
{
	GRAIN_NO_MOVE = 1,			// Does not fall on collapse state
	GRAIN_NO_DESTROY = 2,		// Can't be erased
	GRAIN_FINE = 4,				// Fine grains
	GRAIN_NO_HARM = 8,			// Not to be harmed
	GRAIN_VALUABLE = 16,		// To be collected
	GRAIN_BEVEL = 32,
	GRAIN_COARSE = 64,
	GRAIN_FLUID = 128,
	GRAIN_REPLICATE = 256,
	GRAIN_FUND = 512,
	GRAIN_EXPLOSIVE = 1024
};
// GRAIN_NO_MOVE, GRAIN_NO_DESTROY grains lose GRAIN_NO_MOVE state on loss of health

enum GAMESTATES
{
	STATE_SPLASH = 0,
	STATE_MENU,
	STATE_SINGLE,
	STATE_CAMPAIGN,
	STATE_SELECTCAMPAIGN,
	STATE_CREDITS,
	STATE_OPTIONS,
	STATE_CONFIRM,
	STATE_INGAME,
};

enum INGAMESTATES
{
	STATE_PLAYING = 0,
	STATE_CHECKING,
	STATE_WON,
	STATE_LOST
};

enum MOUSESTATES
{
	MOUSE_IDLE = 0,
	MOUSE_DIRECTION,
	MOUSE_RADIAL
};

class Campaigns
{
public:
	const char *name;
	const char *filename;
};

class Level
{
public:
	SDL_Surface *thumb;
	const char *name;
	const char *filename;
	const char *desc;
	const char *levelfile;
	const char *bgfile;
	const char *skinfile;
	const char *accuracyfile;
	const char *blockfile;
	int replay_exists;
	int money;
	int drill[5];
	int bomb[5];
	int drillcount[5];
	int bombcount[5];
	float valuables;
	float noharm;
	float clearpercent;
	int pass;
	int time;
	int x1,y1,x2,y2;
	int flagx, flagy;
	int unlockscore;
	int hiscore;
	float score_pass;
	float score_valuablepercent;
	float score_noharmpercent;
	float score_timepersec;
	float score_budget;
	float score_accuracymax;
	float score_accuracypenalty;
	Level *next;
	Level();
	~Level();
};

class StateRepository
{
public:
	StateRepository() { string = NULL; next = NULL; }
	~StateRepository() { delete[] string; }
	char *string;
	int value;
	StateRepository *next;
};

class Grain
{
public:
	int color;
	int type;
	int hits;
};

class Drill
{
public:
	float x, y;
	float direction;
	int ttl;
	int drilltype;
	int bombtype;
	int color;
	int soundhandle;
};

struct DrillTypes
{
	int drillstrength;
	int cost;
	float bias;
	float shaky;
	float misdirection;
};

struct BombTypes
{
	int explosionstrength;
	int explosionrange;
	int eiterations;
	int ebranches;
	int shakes;
	int color;
	int cost;
	int piterations;
	int pbranches;
	int vrange;
	int type;
};

struct GrainTypes
{
	int hits;
	int type;
	int colorkey;
	SDL_Surface *texture;
};

struct MouseRegion
{
	int x,y,w,h;
	int id;
};

struct Coords
{
	int x, y;
};

struct MainMenuStruct
{
	Coords pos[6];
};

struct CreditsStruct
{
	Coords pos;
};

struct OptionsStruct
{
	Coords pos;
};

struct JukeboxStruct
{
	Coords pos[4];
};

struct SingleStruct
{
	Coords pos[5];
};

struct CareerStruct
{
	Coords pos;
};

struct MapStruct
{
	Coords pos[9];
};

struct ReplayEvent
{
	int physicstick;
	int x, y, direction;
	int drill, bomb, fuse;
};

extern ReplayEvent replaybuffer[1024]; // 1024 bombs should be enough for everyone
extern int gReplayIndex;
extern int gPhysicsTick;
extern int gReplayMode;

extern MainMenuStruct MainMenu;
extern CreditsStruct CreditsMenu;
extern OptionsStruct OptionsMenu;
extern JukeboxStruct JukeboxMenu;
extern SingleStruct SingleMenu;
extern CareerStruct CareerMenu;
extern MapStruct MapMenu;

class DelayedDig
{
public:
	int x, y, r, pow;
	DelayedDig *next;
};

void shake();
void collapse();

extern CFL *cfl;


extern char * gCreditsText;


extern DelayedDig *gDelayedDigRoot;
extern StateRepository *gStateRoot;
extern Level *gCurrentLevel;
extern Level *gLevelList;
extern int gCurrentMoney;
extern int gTotalValuables;
extern int gCurrentValuables;
extern int gTotalNoharm;
extern int gCurrentNoharm;
extern int gCurrentDugTotal;
extern int gGrainTypes;
extern int gReplicateParticle;

extern int gActiveBomb;
extern int gActiveDrill;
extern int gFuseLength;
extern int gCheckActive;
extern int gMouseIsDown;


extern int gSmokeCap;
extern int gExplosionCap;
extern int gSparkleCap;
extern int gFPSCap;
extern int gEnableLittleTouches;
extern int gPerfInfo;
extern int gCurrentCampaign;
extern int gCampaignScore;

extern int gStateStartTick;
extern int gTextStartTick;
extern int gMouseActive;
extern int gMouseOver;
extern int gMouseDown;
extern int gMouseState;
extern int gPhysicsTick;
extern int gNextGraphicsTick;
extern int gLastTick;

extern int mousex, mousey;
extern int gGameState;
extern int gIngameState;
extern int gShakes;
extern int grainmap[];
extern Grain particle[];
extern Drill drill[];
extern DrillTypes *drillinfo;
extern BombTypes *bombinfo;
extern GrainTypes *graininfo;
extern MouseRegion *mouseinfo;
extern Campaigns *campaigninfo;
extern int gCampaignCount;

extern MersenneTwister prand;
extern MersenneTwister vrand;

extern SDL_Surface *gFlag;
extern SDL_Surface *gMap;

extern SDL_Surface *gScreen;
extern SDL_Surface *gDrills;
extern SDL_Surface *gBombs;
extern SDL_Surface *gBackground;
extern SDL_Surface *gCommands, *gCommandsHot, *gCommandsActive, *gCommandsDown;
extern SDL_Surface *gCursors;
extern SDL_Surface *gDirection;
extern SDL_Surface *gRadials;
extern SDL_Surface *gFont;
extern SDL_Surface *gBigFont;
extern SDL_Surface *gMenubg;
extern SDL_Surface *gSplash;
extern SDL_Surface *gCheckLayer;
extern SDL_Surface *gBlockMask;
extern SDL_Surface *gFFTTexture;

void drillrect(int x, int y, int width, int height, int pow);
void drillcircle(int x, int y, int r, int pow);
void drawcircle(int x, int y, int r, int c, int type);
void drawcirclez(int x, int y, int r, int c, int type);
void drawcirclecheck(int x, int y, int r, int c, int type);
void drawrect(int x, int y, int width, int height, int c);
void drawrectb(int x, int y, int width, int height, int c);
void fullblit(SDL_Surface *surf, int x1, int y1, int w1, int h1, int x2, int y2, int w2, int h2);
void alphablit(SDL_Surface *surf, int x1, int y1, int w1, int h1, int x2, int y2);
void alphaburn(SDL_Surface *surf, int x1, int y1, int w1, int h1, int x2, int y2);
void blitrect(int x, int y, int w, int h, int color);
void blitradials(int x, int y, int cursor);
void blitdirection(int x, int y, int cursor);
void blitcursor(int x, int y, int cursor);
void blit(SDL_Surface *surf, int x, int y, int w, int h);
void blitpiece(SDL_Surface *surf, int x, int y, int w, int h);
void blitstring(int x, int y, const char *text);
void blitstring32(int x, int y, const char *text);
void init();
void bloom();
void drawdescription(int x, int y, int w, const char *desc, int lastchar);

Level * parselevel(const char *filename);

unsigned int blend_add(unsigned int source, unsigned int target);
unsigned int blend_mul(unsigned int source, unsigned int target);
//unsigned int blend_avg(unsigned int source, unsigned int target);
#define blend_avg(source, target) ((((source) & 0xfcfcfc) + ((target) & 0xfcfcfc)) >> 1)

int ingame_events();
int ingame_render();

int splash_events();
void splash_render();

int menu_events();
void menu_render();
void single_render();
void credits_render();
void options_render();
void selectcampaign_render();
void campaign_render();
void confirm_render();

int textbutton(int id, int x, int y, const char *text);

void changestate(int newstate);
void reset(Level *aLevel);

void add_sparkle(int x, int y);
int add_explosion(int x, int y, int radius, int iterations, int children);
void reset_particles();
void render_particles();
void particle_physics();

extern int gActiveSparkles;
extern int gActiveExplosions;
extern int gActiveSmokes;

extern int gMusicVolume;
extern int gSfxVolume;
extern int gFmodTuneChannel;

void load_campaign(const char *filename);

void serialize_states();
void deserialize_states();
int read_state(const char * string);
void set_state(const char *string, int value);
void wipe_states();

void refresh_levels();
int checkmousehit(int x, int y, int w, int h);
int replay_exists(Level *l);
void read_replay(Level *l);