#include <stdio.h>
#include <io.h>
#include "CFLLibrary.h"
#include "CFLDirectoryEntry.h"
#include "charge.h"

MainMenuStruct MainMenu; // 6
CreditsStruct CreditsMenu; // 1
OptionsStruct OptionsMenu; // 1
JukeboxStruct JukeboxMenu; // 4
SingleStruct SingleMenu; // 5
CareerStruct CareerMenu; // 1
MapStruct MapMenu; // 9



void find_old(const char *oldlevelname)
{
	Level * walker;
	if (oldlevelname)
	{
		walker = gLevelList;
		while (walker && _stricmp(walker->filename, oldlevelname) != 0)
		{
			walker = walker->next;
		}
		free((void*)oldlevelname);
		if (walker)
			gCurrentLevel = walker;
		else
			gCurrentLevel = gLevelList;
	}
	else
	{
		gCurrentLevel = gLevelList;
	}
}

void refresh_levels()
{
	const char * oldlevelname = NULL;
	if (gCurrentLevel)
	{
		oldlevelname = _strdup(gCurrentLevel->filename);
	}
	Level *walker = gLevelList;
	while (walker)
	{
		Level *l = walker;
		walker = walker->next;
		delete l;
	}
	gLevelList = NULL;

	// gather stuff from CFL
	CFLDirectoryEntry * cflwalker = cfl->mRootDir;
	while (cflwalker)
	{
		if (strstr(cflwalker->mName, ".lev") != NULL)
		{
			Level *l = parselevel(cflwalker->mName);
			l->next = gLevelList;
			gLevelList = l;
		}
		cflwalker = cflwalker->mNext;
	}

	// check files on disk
	long handle;
	struct _finddata_t findBuffer;
	handle = (long)_findfirst("*.lev",&findBuffer);
	if (handle >= 0)
	do
	{
		// is it already in the list?
		Level * lw = gLevelList;
		while (lw && _stricmp(lw->filename, findBuffer.name) != 0)
		{
			lw = lw->next;
		}
		// new one
		if (!lw)
		{
			Level *l = parselevel(findBuffer.name);
			l->next = gLevelList;
			gLevelList = l;
		}
	}
	while (_findnext(handle,&findBuffer) == 0);

	find_old(oldlevelname);
}

void load_campaign(const char *filename)
{
	const char * oldlevelname = NULL;
	if (gCurrentLevel)
	{
		oldlevelname = _strdup(gCurrentLevel->filename);
	}

	TiXmlDocument doc(filename);
	bool loadOkay = doc.LoadFile();
	
	gCampaignScore = 0;

	if (!loadOkay)
	{
		free((void*)oldlevelname);
		return;
	}

	Level *lw = gLevelList;
	while (lw)
	{
		Level *l = lw;
		lw = lw->next;
		delete l;
	}
	gLevelList = NULL;

	TiXmlElement * root = doc.FirstChildElement("campaign");
	TiXmlElement * walker = root->FirstChildElement();
	while (walker)
	{
		const char * tag = walker->Value();

		if (_stricmp(tag, "img") == 0)
		{
			int len;
			char * data = RAMfget(walker->Attribute("file"), len);
			SDL_RWops * rw = SDL_RWFromMem(data, len);
			SDL_Surface *temp = IMG_Load_RW(rw, 1); // frees the data
			SDL_Surface *t = SDL_ConvertSurface(temp, gScreen->format, SDL_SWSURFACE);
			SDL_FreeSurface(temp);
			if (strtol(walker->Attribute("usecolorkey"),NULL,10))
			{
				SDL_SetColorKey(t, SDL_SRCCOLORKEY | SDL_RLEACCEL, strtol(walker->Attribute("color"),NULL,16));
			}
			const char * img = walker->Attribute("name");
			if (_stricmp(img, "flag") == 0)
			{
				if (gFlag)
					SDL_FreeSurface(gFlag);
				gFlag = t;
			}
			if (_stricmp(img, "map") == 0)
			{
				if (gMap)
					SDL_FreeSurface(gMap);
				gMap = t;
			}
		}
		else
		if (_stricmp(tag, "levels") == 0)
		{
			int count = strtol(walker->Attribute("count"),NULL,10);
			int i;
			TiXmlElement * sub = walker->FirstChildElement();
			for (i = 0; i < count; i++)
			{
				Level *l = parselevel(sub->Attribute("file"));
				l->next = gLevelList;
				gLevelList = l;
				l->flagx = strtol(sub->Attribute("flagx"),NULL,10);
				l->flagy = strtol(sub->Attribute("flagy"),NULL,10);
				l->unlockscore = strtol(sub->Attribute("unlockscore"),NULL,10);

				gCampaignScore += l->hiscore; // should be loaded in parselevel
				
				sub = sub->NextSiblingElement();
			}
			// reverse list
			Level *l = gLevelList;
			gLevelList = NULL;
			while (l)
			{
				Level *ln = l->next;
				l->next = gLevelList;
				gLevelList = l;
				l = ln;
			}
		}

		walker = walker->NextSiblingElement();
	}

	if (oldlevelname)
	{
		find_old(oldlevelname);
	}
	else
	{
		gCurrentLevel = gLevelList;	
		// select the level that has no hi score at the moment
		while (gCurrentLevel->hiscore > 0 && gCurrentLevel->next)
			gCurrentLevel = gCurrentLevel->next;
	}

	set_state(filename, gCampaignScore);
}

Level * parselevel(const char *filename)
{
	TiXmlDocument doc(filename);
	bool loadOkay = doc.LoadFile();
	
	if (!loadOkay)
	{
		return NULL;
	}

	Level *l = new Level();

	l->filename = _strdup(filename);
	
	TiXmlElement * root = doc.FirstChildElement("level");
	TiXmlElement * walker = root->FirstChildElement();
	while (walker)
	{
		const char * tag = walker->Value();
		if (_stricmp(tag,"level") == 0)		
			l->levelfile = _strdup(walker->Attribute("file"));
		else
		if (_stricmp(tag,"background") == 0)
			l->bgfile = _strdup(walker->Attribute("file"));
		else
		if (_stricmp(tag,"accuracy") == 0)
			l->accuracyfile = _strdup(walker->Attribute("file"));
		else
		if (_stricmp(tag,"skin") == 0)
			l->skinfile = _strdup(walker->Attribute("file"));
		else
		if (_stricmp(tag,"block") == 0)
			l->blockfile = _strdup(walker->Attribute("file"));
		else
		if (_stricmp(tag,"thumb") == 0)
		{
			int len;
			char * data = RAMfget(walker->Attribute("file"), len);
			SDL_RWops * rw = SDL_RWFromMem(data, len);
			SDL_Surface *temp = IMG_Load_RW(rw, 1); // frees the data
			if (temp)
				l->thumb = SDL_ConvertSurface(temp, gScreen->format, SDL_SWSURFACE);
			else
				l->thumb = NULL;
			SDL_FreeSurface(temp);
		}
		else
		if (_stricmp(tag,"desc") == 0)
			l->desc = _strdup(walker->Attribute("text"));
		else
		if (_stricmp(tag,"name") == 0)
			l->name = _strdup(walker->Attribute("text"));
		else
		if (_stricmp(tag,"budget") == 0)
		{
			l->money = strtol(walker->Attribute("money"),NULL,10);
		}
		else
		if (_stricmp(tag,"goals") == 0)
		{
			l->valuables = (float)strtod(walker->Attribute("valuables"),NULL);
			l->noharm = (float)strtod(walker->Attribute("noharm"),NULL);
			l->pass = strtol(walker->Attribute("pass"),NULL,10);
			l->time = strtol(walker->Attribute("time"),NULL,10);
			l->clearpercent = 0.9f;
			if (walker->Attribute("clearpercent"))
				l->clearpercent = (float)strtod(walker->Attribute("clearpercent"),NULL);
		}
		else
		if (_stricmp(tag,"pass") == 0)
		{
			l->x1 = strtol(walker->Attribute("x1"),NULL,10);
			l->y1 = strtol(walker->Attribute("y1"),NULL,10);
			l->x2 = strtol(walker->Attribute("x2"),NULL,10);
			l->y2 = strtol(walker->Attribute("y2"),NULL,10);
		}
		else
		if (_stricmp(tag, "score") == 0)
		{
			l->score_pass = (float)strtod(walker->Attribute("pass"),NULL);
			l->score_valuablepercent = (float)strtod(walker->Attribute("valuablepercent"),NULL);
			l->score_noharmpercent = (float)strtod(walker->Attribute("noharmpercent"),NULL);
			l->score_timepersec = (float)strtod(walker->Attribute("timepersec"),NULL);
			l->score_budget = (float)strtod(walker->Attribute("budget"),NULL);
			l->score_accuracymax = (float)strtod(walker->Attribute("maxaccuracy"),NULL);
			l->score_accuracypenalty = (float)strtod(walker->Attribute("accuracypenalty"),NULL);
		}	
		else
		if (_stricmp(tag, "drillconfig") == 0)
		{
			int count = 5;
			int i;
			TiXmlElement * sub = walker->FirstChildElement();
			for (i = 0; i < count; i++)
			{
				l->drill[i] = strtol(sub->Attribute("number"),0,10);
				l->drillcount[i] = strtol(sub->Attribute("available"),0,10);
				
				sub = sub->NextSiblingElement();
			}
		}
		else
		if (_stricmp(tag, "bombconfig") == 0)
		{
			int count = 5;
			int i;
			TiXmlElement * sub = walker->FirstChildElement();
			for (i = 0; i < count; i++)
			{
				l->bomb[i] = strtol(sub->Attribute("number"),0,10);
				l->bombcount[i] = strtol(sub->Attribute("available"),0,10);
				
				sub = sub->NextSiblingElement();
			}
		}


		walker = walker->NextSiblingElement();
	}

	l->hiscore = read_state(filename);
	l->replay_exists = replay_exists(l);

	return l;
}

int find_grain(int color)
{
	int i;
	for (i = 0; i < gGrainTypes; i++)
		if (color == graininfo[i].colorkey)
			return i+1;
	return 0;
}

void reset(Level *aLevel)
{
	gCurrentMoney = aLevel->money;
	gIngameState = STATE_PLAYING;
	reset_particles();

	int len;
	char * data = RAMfget(aLevel->levelfile, len);
	SDL_RWops * rw = SDL_RWFromMem(data, len);
	SDL_Surface *temp = IMG_Load_RW(rw, 1); // frees the data
	SDL_Surface *level = SDL_ConvertSurface(temp, gScreen->format, SDL_SWSURFACE);
	SDL_FreeSurface(temp);

	if (gCheckLayer)
	{
		SDL_FreeSurface(gCheckLayer);
		gCheckLayer = NULL;
	}
	if (gBackground)
	{
		SDL_FreeSurface(gBackground);
		gBackground = NULL;
	}
	if (gBlockMask)
	{
		SDL_FreeSurface(gBlockMask);
		gBlockMask = NULL;
	}
	if (aLevel->bgfile)
	{
		int len;
		char * data = RAMfget(aLevel->bgfile, len);
		SDL_RWops * rw = SDL_RWFromMem(data, len);
		temp = IMG_Load_RW(rw, 1); // frees the data
		if (temp)
		{
			gBackground = SDL_ConvertSurface(temp, gScreen->format, SDL_SWSURFACE);
			SDL_FreeSurface(temp);
		}
	}
	if (aLevel->accuracyfile)
	{
		int len;
		char * data = RAMfget(aLevel->accuracyfile, len);
		SDL_RWops * rw = SDL_RWFromMem(data, len);
		temp = IMG_Load_RW(rw, 1); // frees the data
		if (temp)
		{
			gCheckLayer = SDL_ConvertSurface(temp, gScreen->format, SDL_SWSURFACE);
			SDL_FreeSurface(temp);
		}
	}
	if (aLevel->blockfile)
	{
		int len;
		char * data = RAMfget(aLevel->blockfile, len);
		SDL_RWops * rw = SDL_RWFromMem(data, len);
		temp = IMG_Load_RW(rw, 1); // frees the data
		if (temp)
		{
			gBlockMask = SDL_ConvertSurface(temp, gScreen->format, SDL_SWSURFACE);
			SDL_FreeSurface(temp);
		}		
	}

	if (gCheckLayer == NULL)
	{
		gCheckLayer = SDL_CreateRGBSurface(SDL_SWSURFACE, 
										   680, 
										   600, 
										   gScreen->format->BitsPerPixel, 
										   gScreen->format->Rmask,
										   gScreen->format->Gmask,
										   gScreen->format->Bmask,
										   gScreen->format->Amask);
	}

	SDL_Surface *skin = NULL;
	if (aLevel->skinfile)
	{
		int len;
		char * data = RAMfget(aLevel->skinfile, len);
		SDL_RWops * rw = SDL_RWFromMem(data, len);
		temp = IMG_Load_RW(rw, 1); // frees the data
		if (temp)
		{
			skin = SDL_ConvertSurface(temp, gScreen->format, SDL_SWSURFACE);
			SDL_FreeSurface(temp);
		}
	}

	// Lock surface if needed
	if (SDL_MUSTLOCK(level))
		if (SDL_LockSurface(level) < 0) 
			return;

	if (skin && SDL_MUSTLOCK(skin))
		if (SDL_LockSurface(skin) < 0) 
			return;

	gTotalValuables = 0;
	gCurrentValuables = 0;
	gTotalNoharm = 0;
	gCurrentNoharm = 0;
	gReplicateParticle = 0;
	gMouseIsDown = 0;
	gCurrentDugTotal = 0;
	gReplayIndex = 0;
	gPhysicsTick = 0;

	prand.init_genrand(0xc0cac01a);	

	gCheckActive = 0;
	if (aLevel->accuracyfile || gCurrentLevel->pass > 0)
		gCheckActive = 1; // start with 'check' active always if there's something to see

	int i, j, c = 0;
	for (i = 0; i < 600*680; i++)
		particle[i].hits = UNUSED_PARTICLE;
	for (i = 0; i < 600; i++)
	{
		int spofs = i * level->pitch / 4;
		for (j = 0; j < 680; j++, c++)
		{
			int color = find_grain(((unsigned int*)level->pixels)[spofs + j]);
			if (color == 0)
			{
				grainmap[c] = -1;
			}
			else
			{
				grainmap[c] = c;
				particle[c].hits = 0;
				int id = color - 1;
				particle[c].type = graininfo[id].type;
				particle[c].hits = graininfo[id].hits;

				if (particle[c].type & GRAIN_VALUABLE)
					gTotalValuables++;
				if (particle[c].type & GRAIN_NO_HARM)
					gTotalNoharm++;

				if (skin)
				{
					particle[c].color = ((unsigned int*)skin->pixels)[spofs + j];
				}
				else
				{
					particle[c].color = ((unsigned int*)graininfo[id].texture->pixels)[(i % graininfo[id].texture->w) * graininfo[id].texture->pitch / 4 + j % graininfo[id].texture->h];
				}
			}
		}
	}	
	
	if (!skin)
	{
		for (i = 0; i < 600; i++)
		{
			c = i * 680;
			for (j = 0; j < 680; j++, c++)
			{
				if (grainmap[c] >= 0 && particle[grainmap[c]].type & GRAIN_BEVEL)
				{
					if (i > 0 && j > 0 && (grainmap[c - 681] < 0 || particle[grainmap[c]].type != particle[grainmap[c - 681]].type))
					{
						// nw
						particle[grainmap[c]].color = blend_add(particle[grainmap[c]].color, 0x3f3f3f);
					}
					else
					if (i > 1 && j > 1 && (grainmap[c - 681*2] < 0 || particle[grainmap[c]].type != particle[grainmap[c - 681*2]].type))
					{
						// nw
						particle[grainmap[c]].color = blend_add(particle[grainmap[c]].color, 0x3f3f3f);
					}
					else
					if (i > 2 && j > 2 && (grainmap[c - 681*3] < 0 || particle[grainmap[c]].type != particle[grainmap[c - 681*3]].type))
					{
						// nw
						particle[grainmap[c]].color = blend_add(particle[grainmap[c]].color, 0x3f3f3f);
					}
					else
					if (i < 599 && j < 679 && (grainmap[c + 681] < 0 || particle[grainmap[c]].type != particle[grainmap[c + 681]].type))
					{
						// sw
						particle[grainmap[c]].color = blend_mul(particle[grainmap[c]].color, 0xafafaf);
					}
					else
					if (i < 598 && j < 678 && (grainmap[c + 681*2] < 0 || particle[grainmap[c]].type != particle[grainmap[c + 681*2]].type))
					{
						// sw
						particle[grainmap[c]].color = blend_mul(particle[grainmap[c]].color, 0xafafaf);
					}
					else
					if (i < 597 && j < 677 && (grainmap[c + 681*3] < 0 || particle[grainmap[c]].type != particle[grainmap[c + 681*3]].type))
					{
						// sw
						particle[grainmap[c]].color = blend_mul(particle[grainmap[c]].color, 0xafafaf);
					}
				}
			}
		}
	}

	// Unlock if needed
    if (SDL_MUSTLOCK(level)) 
        SDL_UnlockSurface(level);

	// Unlock if needed
    if (skin && SDL_MUSTLOCK(skin)) 
        SDL_UnlockSurface(skin);


	for (i = 0; i < 64; i++)
	{
		drill[i].drilltype = -1;
		drill[i].bombtype = -1;
	}

	gShakes = 0;
	gActiveDrill = 1;
	gActiveBomb = 1;
	gFuseLength = 1;

	DelayedDig * dd = gDelayedDigRoot;
	while (dd)
	{
		DelayedDig *d = dd;
		dd = dd->next;
		delete d;
	}
	gDelayedDigRoot = NULL;

	SDL_FreeSurface(level);
	SDL_FreeSurface(skin);
}

void init()
{
	deserialize_states();

	TiXmlDocument doc("charge.xml");
	bool loadOkay = doc.LoadFile();
	
	if (!loadOkay)
	{
		exit(-1);
	}
	
	TiXmlElement * root = doc.FirstChildElement("config");
	TiXmlElement * walker = root->FirstChildElement();
	while (walker)
	{
		const char * tag = walker->Value();
		if (_stricmp(tag,"performance") == 0)
		{
			gExplosionCap = strtol(walker->Attribute("explosioncap"),NULL,10);
			gSmokeCap = strtol(walker->Attribute("smokecap"),NULL,10);
			gSparkleCap = strtol(walker->Attribute("sparklecap"),NULL,10);
			gFPSCap = strtol(walker->Attribute("fpscap"),NULL,10);
			gEnableLittleTouches = strtol(walker->Attribute("littletouches"),NULL,10);
			gPerfInfo = strtol(walker->Attribute("perfinfo"),NULL,10);

		}
		else
		if (_stricmp(tag, "drillconfig") == 0)
		{
			int count = strtol(walker->Attribute("count"),NULL,10);
			int i;
			drillinfo = new DrillTypes[count];
			TiXmlElement * sub = walker->FirstChildElement();
			for (i = 0; i < count; i++)
			{
				int len;
				char * data = RAMfget(sub->Attribute("sound"),len);
				//drillinfo[i].drillsound = FSOUND_Sample_Load(FSOUND_FREE, data, FSOUND_LOOP_NORMAL | FSOUND_LOADMEMORY, 0, len);
				free(data);

				drillinfo[i].drillstrength = strtol(sub->Attribute("drillstrength"),0,10);
				drillinfo[i].cost = strtol(sub->Attribute("cost"),0,10);
				drillinfo[i].bias = (float)strtod(sub->Attribute("bias"),NULL);
				drillinfo[i].shaky = (float)strtod(sub->Attribute("shaky"),NULL);
				drillinfo[i].misdirection = (float)strtod(sub->Attribute("misdirection"),NULL);
				
				sub = sub->NextSiblingElement();
			}
		}
		else
		if (_stricmp(tag, "bombconfig") == 0)
		{
			int count = strtol(walker->Attribute("count"),NULL,10);
			int i;
			bombinfo = new BombTypes[count];
			TiXmlElement * sub = walker->FirstChildElement();
			for (i = 0; i < count; i++)
			{
				int len;
				char * data = RAMfget(sub->Attribute("sound"),len);
				//bombinfo[i].explosionsound = FSOUND_Sample_Load(FSOUND_FREE, data, FSOUND_LOOP_OFF | FSOUND_LOADMEMORY, 0, len);
				free(data);

				bombinfo[i].explosionstrength = strtol(sub->Attribute("estrength"),0,10);
				bombinfo[i].explosionrange = strtol(sub->Attribute("erange"),0,10);
				bombinfo[i].eiterations = strtol(sub->Attribute("eiter"),0,10);
				bombinfo[i].ebranches = strtol(sub->Attribute("ebranch"),0,10);
				bombinfo[i].shakes = strtol(sub->Attribute("shakes"),0,10);
				bombinfo[i].color = strtol(sub->Attribute("color"),0,16);
				bombinfo[i].cost = strtol(sub->Attribute("cost"),0,10);
				bombinfo[i].piterations = strtol(sub->Attribute("viter"),0,10);
				bombinfo[i].pbranches = strtol(sub->Attribute("vbranch"),0,10);
				bombinfo[i].vrange = strtol(sub->Attribute("vrange"),0,10);
				bombinfo[i].type = strtol(sub->Attribute("type"),0,10);
				
				sub = sub->NextSiblingElement();
			}
		}
		else
		if (_stricmp(tag, "grainconfig") == 0)
		{
			int count = strtol(walker->Attribute("count"),NULL,10);
			gGrainTypes = count;
			int i;
			graininfo = new GrainTypes[count];
			TiXmlElement * sub = walker->FirstChildElement();
			for (i = 0; i < count; i++)
			{
				graininfo[i].type = 0;
				graininfo[i].hits = strtol(sub->Attribute("hits"),NULL,10);
				graininfo[i].colorkey = strtol(sub->Attribute("colorkey"),NULL,16);

				graininfo[i].type |= (!!strtol(sub->Attribute("nomove"),NULL,10)) * GRAIN_NO_MOVE;
				graininfo[i].type |= (!!strtol(sub->Attribute("nodestroy"),NULL,10)) * GRAIN_NO_DESTROY;
				graininfo[i].type |= (!!strtol(sub->Attribute("valuable"),NULL,10)) * GRAIN_VALUABLE;
				graininfo[i].type |= (!!strtol(sub->Attribute("noharm"),NULL,10)) * GRAIN_NO_HARM;
				graininfo[i].type |= (!!strtol(sub->Attribute("bevel"),NULL,10)) * GRAIN_BEVEL;
				graininfo[i].type |= (!!strtol(sub->Attribute("replicate"),NULL,10)) * GRAIN_REPLICATE;
				graininfo[i].type |= (!!strtol(sub->Attribute("fund"),NULL,10)) * GRAIN_FUND;
				graininfo[i].type |= (!!strtol(sub->Attribute("explosive"),NULL,10)) * GRAIN_EXPLOSIVE;

				int t = strtol(sub->Attribute("roughness"),NULL,10);
				switch (t)
				{
				case 0:
					graininfo[i].type |= GRAIN_COARSE;
					break;
				case 1:
					// normal
					break;
				case 2:
					graininfo[i].type |= GRAIN_FINE;
					break;
				case 3:
					graininfo[i].type |= GRAIN_FLUID;
					break;
				}

				int len;
				char * data = RAMfget(sub->Attribute("texture"), len);
				SDL_RWops * rw = SDL_RWFromMem(data, len);
				SDL_Surface *temp = IMG_Load_RW(rw, 1); // frees the data
				graininfo[i].texture = SDL_ConvertSurface(temp, gScreen->format, SDL_SWSURFACE);
				SDL_FreeSurface(temp);
				
				sub = sub->NextSiblingElement();
			}
		}
		else
		if (_stricmp(tag, "img") == 0)
		{
			int len;
			char * data = RAMfget(walker->Attribute("file"), len);
			SDL_RWops * rw = SDL_RWFromMem(data, len);
			SDL_Surface *temp = IMG_Load_RW(rw, 1); // frees the data
			SDL_Surface *t = temp; //SDL_ConvertSurface(temp, gScreen->format, SDL_SWSURFACE);
			if (t->format->BitsPerPixel < 32)
			{
				t = SDL_ConvertSurface(temp, gScreen->format, SDL_SWSURFACE);
				SDL_FreeSurface(temp);
			}
			
			if (strtol(walker->Attribute("usecolorkey"),NULL,10))
			{
				SDL_SetColorKey(t, SDL_SRCCOLORKEY | SDL_RLEACCEL, strtol(walker->Attribute("color"),NULL,16));
			}
			const char * img = walker->Attribute("name");
			if (_stricmp(img, "cmd") == 0)
				gCommands = t;
			if (_stricmp(img, "cmdhot") == 0)
				gCommandsHot = t;
			if (_stricmp(img, "cmddown") == 0)
				gCommandsDown = t;
			if (_stricmp(img, "cmdactive") == 0)
				gCommandsActive = t;
			if (_stricmp(img, "cursors") == 0)
				gCursors = t;
			if (_stricmp(img, "direction") == 0)
				gDirection = t;
			if (_stricmp(img, "radials") == 0)
				gRadials = t;
			if (_stricmp(img, "font") == 0)
				gFont = t;
			if (_stricmp(img, "bigfont") == 0)
				gBigFont = t;
			if (_stricmp(img, "menubg") == 0)
				gMenubg = t;
			if (_stricmp(img, "splash") == 0)
				gSplash = t;
			if (_stricmp(img, "drills") == 0)
				gDrills = t;
			if (_stricmp(img, "bombs") == 0)
				gBombs = t;
			if (_stricmp(img, "fft") == 0)
				gFFTTexture = t;
		}
		else
		if (_stricmp(tag, "buttonconfig") == 0)
		{
			int count = strtol(walker->Attribute("count"),NULL,10);
			int i;
			mouseinfo = new MouseRegion[count];
			TiXmlElement * sub = walker->FirstChildElement();
			for (i = 0; i < count; i++)
			{
				mouseinfo[i].id = 0;
				mouseinfo[i].x = strtol(sub->Attribute("x"), NULL, 10);
				mouseinfo[i].y = strtol(sub->Attribute("y"), NULL, 10);
				mouseinfo[i].w = strtol(sub->Attribute("w"), NULL, 10);
				mouseinfo[i].h = strtol(sub->Attribute("h"), NULL, 10);
				if (_stricmp(sub->Attribute("name"), "drill1") == 0)
					mouseinfo[i].id = 1;
				if (_stricmp(sub->Attribute("name"), "drill2") == 0)
					mouseinfo[i].id = 2;
				if (_stricmp(sub->Attribute("name"), "drill3") == 0)
					mouseinfo[i].id = 3;
				if (_stricmp(sub->Attribute("name"), "drill4") == 0)
					mouseinfo[i].id = 4;
				if (_stricmp(sub->Attribute("name"), "drill5") == 0)
					mouseinfo[i].id = 5;
				if (_stricmp(sub->Attribute("name"), "bomb1") == 0)
					mouseinfo[i].id = 6;
				if (_stricmp(sub->Attribute("name"), "bomb2") == 0)
					mouseinfo[i].id = 7;
				if (_stricmp(sub->Attribute("name"), "bomb3") == 0)
					mouseinfo[i].id = 8;
				if (_stricmp(sub->Attribute("name"), "bomb4") == 0)
					mouseinfo[i].id = 9;
				if (_stricmp(sub->Attribute("name"), "bomb5") == 0)
					mouseinfo[i].id = 10;
				if (_stricmp(sub->Attribute("name"), "fuseslider") == 0)
					mouseinfo[i].id = 11;
				if (_stricmp(sub->Attribute("name"), "quit") == 0)
					mouseinfo[i].id = 12;
				if (_stricmp(sub->Attribute("name"), "check") == 0)
					mouseinfo[i].id = 13;

				sub = sub->NextSiblingElement();
			}		
		}
		else
		if (_stricmp(tag, "campaignconfig") == 0)
		{
			gCampaignCount = strtol(walker->Attribute("count"),NULL,10);
			int i;
			campaigninfo = new Campaigns[gCampaignCount];
			TiXmlElement * sub = walker->FirstChildElement();
			for (i = 0; i < gCampaignCount; i++)
			{
				campaigninfo[i].name = _strdup(sub->Attribute("name"));
				campaigninfo[i].filename = _strdup(sub->Attribute("filename"));
				sub = sub->NextSiblingElement();
			}
		}
		else
		if (_stricmp(tag, "credits") == 0)
		{
			gCreditsText = _strdup(walker->Attribute("whoop"));
		}
		else
		if (_stricmp(tag, "sfx") == 0)
		{
			int len;
			char * data = RAMfget(walker->Attribute("file"),len);
			//FSOUND_SAMPLE *s = FSOUND_Sample_Load(FSOUND_FREE, data, FSOUND_LOOP_OFF | FSOUND_LOADMEMORY, 0, len);
			free(data);
			const char * sfx = walker->Attribute("name");
//			if (_stricmp(sfx, "menuclick") == 0)
//				gSampleMenuClick = s;
//			if (_stricmp(sfx, "smallex") == 0)
//				gSampleSmallEx = s;
		}
		else
		if (_stricmp(tag, "menu") == 0)
		{
			Coords *p;
			int count = 0;
			if (_stricmp(walker->Attribute("name"),"main") == 0)
			{
				p = &(MainMenu.pos[0]);
				count = 6;
			}
			else
			if (_stricmp(walker->Attribute("name"),"credits") == 0)
			{
				p = &(CreditsMenu.pos);
				count = 1;
			}
			else
			if (_stricmp(walker->Attribute("name"),"options") == 0)
			{
				p = &(OptionsMenu.pos);
				count = 1;
			}
			else
			if (_stricmp(walker->Attribute("name"),"jukebox") == 0)
			{
				p = &(JukeboxMenu.pos[0]);
				count = 4;
			}
			else
			if (_stricmp(walker->Attribute("name"),"single") == 0)
			{
				p = &(SingleMenu.pos[0]);
				count = 5;
			}
			else
			if (_stricmp(walker->Attribute("name"),"Career") == 0)
			{
				p = &(CareerMenu.pos);
				count = 1;
			}
			else
			if (_stricmp(walker->Attribute("name"),"map") == 0)
			{
				p = &(MapMenu.pos[0]);
				count = 9;
			}
			
			int i = 0;
			TiXmlElement * sub = walker->FirstChildElement();
			while (sub && i < count)
			{
				p[i].x = strtol(sub->Attribute("x"),NULL,10);
				p[i].y = strtol(sub->Attribute("y"),NULL,10);
				i++;
				sub = sub->NextSiblingElement();
			}
		}

		walker = walker->NextSiblingElement();
	}

	SDL_WarpMouse(400,300);
	SDL_ShowCursor(0);
	SDL_WM_SetCaption(TITLE,NULL);

	gMusicVolume = read_state("musicvolume") - 1;
	if (gMusicVolume == -1)
		gMusicVolume = 50;

	gSfxVolume = read_state("sfxvolume") - 1;
	if (gSfxVolume == -1)
		gSfxVolume = 50;
}
