#include "charge.h"

/* // reimplemented as a macro
unsigned int blend_avg(unsigned int source, unsigned int target)
{
	return ((source & 0xfcfcfc) + (target & 0xfcfcfc)) >> 1;
}
*/

unsigned int blend_mul(unsigned int source, unsigned int target)
{
  unsigned int sourcer = (source >>  0) & 0xff;
  unsigned int sourceg = (source >>  8) & 0xff;
  unsigned int sourceb = (source >> 16) & 0xff;
  unsigned int targetr = (target >>  0) & 0xff;
  unsigned int targetg = (target >>  8) & 0xff;
  unsigned int targetb = (target >> 16) & 0xff;

  targetr = (sourcer * targetr) >> 8;
  targetg = (sourceg * targetg) >> 8;
  targetb = (sourceb * targetb) >> 8;

  return (targetr <<  0) |
         (targetg <<  8) |
         (targetb << 16);
}

unsigned int blend_add(unsigned int source, unsigned int target)
{
	unsigned int t0, t1;
    static const unsigned int signmask = 0x80808080;

    t0 = (source ^ target) & signmask;
    t1 = (source & target) & signmask;

    source &= ~signmask;
    target &= ~signmask;
    source += target;

    t1 |= t0 & source;
    t1 = (t1 << 1) - (t1 >> 7);
    return (source ^ t0) | t1;
}

void blit(SDL_Surface *surf, int x, int y, int w, int h)
{
	SDL_Rect src, dst;
	src.w = w;
	src.h = h;
	src.x = 0;
	src.y = 0;
	dst.w = w;
	dst.h = h;
	dst.x = x;
	dst.y = y;
	SDL_BlitSurface(surf, &src, gScreen, &dst);
}

void fullblit(SDL_Surface *surf, int x1, int y1, int w1, int h1, int x2, int y2, int w2, int h2)
{
	SDL_Rect src, dst;
	src.w = w1;
	src.h = h1;
	src.x = x1;
	src.y = y1;
	dst.w = w2;
	dst.h = h2;
	dst.x = x2;
	dst.y = y2;
	SDL_BlitSurface(surf, &src, gScreen, &dst);
}

static inline unsigned int Scale32(unsigned int scale, unsigned int p) {

	unsigned int ag = (p & 0xFF00FF00) >> 8;
	unsigned int rb =  p & 0x00FF00FF;

	unsigned int sag = scale * ag;
	unsigned int srb = scale * rb;

	sag = sag & 0xFF00FF00;
	srb = srb >> 8 & 0x00FF00FF;

	return sag | srb;
}

#define RBSWAP(a) ((a) & 0xff00ff00) | (((a) & 0xff) << 16) | (((a) >> 16) & 0xff)

static inline unsigned int alphablend(unsigned int src, unsigned int tgt)
{
	int alpha = (tgt >> 24) & 0xff;
	int invalpha = 0xff - alpha;
	tgt = RBSWAP(tgt);

	return Scale32(alpha, tgt) + Scale32(invalpha, src);
}

static inline unsigned int alphaburnpixel(unsigned int src, unsigned int tgt)
{
	int alpha = (tgt >> 24) & 0xff;
	int invalpha = 0xff - alpha;
	
	return Scale32(invalpha, src);
}

void alphablit(SDL_Surface *surf, int x1, int y1, int w1, int h1, int x2, int y2)
{
	SDL_Rect src, dst;
	src.w = w1;
	src.h = h1;
	src.x = x1;
	src.y = y1;
	dst.w = w1;
	dst.h = h1;
	dst.x = x2;
	dst.y = y2;
	//SDL_BlitSurface(surf, &src, gScreen, &dst);

	int i, j;
	for (i = 0; i < h1; i++)
	{
		int to = (y2 + i) * (gScreen->pitch / 4) + x2;
		int fr = (y1 + i) * (surf->pitch / 4) + x1;
		for (j = 0; j < w1; j++, to++, fr++)
		{
			((unsigned int*)gScreen->pixels)[to] =
				alphablend(((unsigned int*)gScreen->pixels)[to], ((unsigned int*)surf->pixels)[fr]);				
		}
	}
}

void alphaburn(SDL_Surface *surf, int x1, int y1, int w1, int h1, int x2, int y2)
{
	SDL_Rect src, dst;
	src.w = w1;
	src.h = h1;
	src.x = x1;
	src.y = y1;
	dst.w = w1;
	dst.h = h1;
	dst.x = x2;
	dst.y = y2;
	//SDL_BlitSurface(surf, &src, gScreen, &dst);

	int i, j;
	for (i = 0; i < h1; i++)
	{
		int to = (y2 + i) * (gScreen->pitch / 4) + x2;
		int fr = (y1 + i) * (surf->pitch / 4) + x1;
		for (j = 0; j < w1; j++, to++, fr++)
		{
			((unsigned int*)gScreen->pixels)[to] =
				alphaburnpixel(((unsigned int*)gScreen->pixels)[to], ((unsigned int*)surf->pixels)[fr]);				
		}
	}
}

void blitpiece(SDL_Surface *surf, int x, int y, int w, int h)
{
	SDL_Rect src, dst;
	src.w = w;
	src.h = h;
	src.x = x;
	src.y = y;
	dst.w = w;
	dst.h = h;
	dst.x = x;
	dst.y = y;
	SDL_BlitSurface(surf, &src, gScreen, &dst);
}

void blitcursor(int x, int y, int cursor)
{
	SDL_Rect src, dst;
	src.w = 64;
	src.h = 64;
	src.x = 0;
	src.y = 64*cursor;
	dst.w = 64;
	dst.h = 64;
	dst.x = x - 32;
	dst.y = y - 32;
	SDL_BlitSurface(gCursors, &src, gScreen, &dst);
}

void blitdirection(int x, int y, int cursor)
{
	SDL_Rect src, dst;
	src.w = 64;
	src.h = 64;
	src.x = 0;
	src.y = 64*cursor;
	dst.w = 64;
	dst.h = 64;
	dst.x = x - 32;
	dst.y = y - 32;
	SDL_BlitSurface(gDirection, &src, gScreen, &dst);
}

void blitradials(int x, int y, int cursor)
{
	SDL_Rect src, dst;
	src.w = 64;
	src.h = 64;
	src.x = 0;
	src.y = 64*cursor;
	dst.w = 64;
	dst.h = 64;
	dst.x = x - 32;
	dst.y = y - 32;
	SDL_BlitSurface(gRadials, &src, gScreen, &dst);
}

void blitchar(int x, int y, char character)
{
	SDL_Rect src, dst;
	src.w = 8;
	src.h = 8;
	src.x = 0;
	src.y = 8*character;
	dst.w = 8;
	dst.h = 8;
	dst.x = x;
	dst.y = y;
	SDL_BlitSurface(gFont, &src, gScreen, &dst);
}

void blitstring(int x, int y, const char *text)
{
	while (text && *text)
	{
		blitchar(x, y, *text-32);
		text++;
		x += 8;
	}
}

void blitchar32(int x, int y, char character)
{
	SDL_Rect src, dst;
	src.w = 32;
	src.h = 32;
	src.x = 0;
	src.y = 32*character;
	dst.w = 32;
	dst.h = 32;
	dst.x = x;
	dst.y = y;
	SDL_BlitSurface(gBigFont, &src, gScreen, &dst);
}

void blitstring32(int x, int y, const char *text)
{
	while (text && *text)
	{
		blitchar32(x, y, *text-32);
		text++;
		x += 32;
	}
}

void blitrect(int x, int y, int w, int h, int color)
{
	SDL_Rect r;
	r.x = x;
	r.y = y;
	r.w = w;
	r.h = h;
	SDL_FillRect(gScreen, &r, color);
}

void drawrect(int x, int y, int width, int height, int c)
{
	int i, j;
	for (i = 0; i < height; i++)
	{
		// vertical clipping: (top and bottom)
		if ((y + i) >= 0 && (y + i) < HEIGHT)
		{
			int len = width;
			int xofs = x;

			// left border
			if (xofs < 0)
			{
				len += xofs;
				xofs = 0;
			}

			// right border
			if (xofs + len >= WIDTH)
			{
				len -= (xofs + len) - WIDTH;
			}
			int ofs = (i + y) * PITCH + xofs;

			// note that len may be 0 at this point, 
			// and no pixels get drawn!
			for (j = 0; j < len; j++)
				((unsigned int*)gScreen->pixels)[ofs + j] = c;
		}
	}
}

void drawrectb(int x, int y, int width, int height, int c)
{
	/*
	char temp[80];
	sprintf(temp,"%x\n",c);
	OutputDebugString(temp);*/
	int i, j;
	for (i = 0; i < height; i++)
	{
		// vertical clipping: (top and bottom)
		if ((y + i) >= 0 && (y + i) < HEIGHT)
		{
			int len = width;
			int xofs = x;

			// left border
			if (xofs < 0)
			{
				len += xofs;
				xofs = 0;
			}

			// right border
			if (xofs + len >= WIDTH)
			{
				len -= (xofs + len) - WIDTH;
			}
			int ofs = (i + y) * PITCH + xofs;

			// note that len may be 0 at this point, 
			// and no pixels get drawn!
			for (j = 0; j < len; j++)
				((unsigned int*)gScreen->pixels)[ofs + j] = blend_avg(c,((unsigned int*)gScreen->pixels)[ofs + j]);
		}
	}
}

void drawcircle(int x, int y, int r, int c, int type)
{
	int i, j;
	for (i = 0; i < 2 * r; i++)
	{
        // vertical clipping: (top and bottom)
		if ((y - r + i) >= 0 && (y - r + i) < HEIGHT)
		{
			int len = (int)sqrt((float)(r * r - (r - i) * (r - i))) * 2;
			int xofs = x - len / 2;

			// left border
			if (xofs < 0)
			{
				len += xofs;
				xofs = 0;
			}

			// right border
			if (xofs + len >= WIDTH)
			{
				len -= (xofs + len) - WIDTH;
			}
			int ofs = (y - r + i) * PITCH + xofs;
			
			// note that len may be 0 at this point, 
			// and no pixels get drawn!
			for (j = 0; j < len; j++)
			{
				switch (type)
				{
				case 1:
					((unsigned int*)gScreen->pixels)[ofs + j] = 
					blend_avg(c, ((unsigned int*)gScreen->pixels)[ofs + j]);
					break;
				case 2:
					((unsigned int*)gScreen->pixels)[ofs + j] = 
					blend_add(c, ((unsigned int*)gScreen->pixels)[ofs + j]);
					break;
				default:
					((unsigned int*)gScreen->pixels)[ofs + j] = c;
					break;
				}
//				((unsigned int*)gScreen->pixels)[ofs + j] = c;
			}
		}
	}
}

void drawcirclecheck(int x, int y, int r, int c, int type)
{
	int i, j;
	for (i = 0; i < 2 * r; i++)
	{
        // vertical clipping: (top and bottom)
		if ((y - r + i) >= 0 && (y - r + i) < 600)
		{
			int len = (int)sqrt((float)(r * r - (r - i) * (r - i))) * 2;
			int xofs = x - len / 2;

			// left border
			if (xofs < 0)
			{
				len += xofs;
				xofs = 0;
			}

			// right border
			if (xofs + len >= 680)
			{
				len -= (xofs + len) - 680;
			}
			int ofs = (y - r + i) * (gCheckLayer->pitch/4) + xofs;
			
			// note that len may be 0 at this point, 
			// and no pixels get drawn!
			for (j = 0; j < len; j++)
			{
				switch (type)
				{
				case 1:
					((unsigned int*)gCheckLayer->pixels)[ofs + j] = 
					blend_avg(c, ((unsigned int*)gCheckLayer->pixels)[ofs + j]);
					break;
				case 2:
					((unsigned int*)gCheckLayer->pixels)[ofs + j] = 
					blend_add(c, ((unsigned int*)gCheckLayer->pixels)[ofs + j]);
					break;
				default:
					((unsigned int*)gCheckLayer->pixels)[ofs + j] = c;
					break;
				}
//				((unsigned int*)gScreen->pixels)[ofs + j] = c;
			}
		}
	}
}


void drawcirclez(int x, int y, int r, int c, int type)
{
	int i, j;
	for (i = 0; i < 2 * r; i++)
	{
        // vertical clipping: (top and bottom)
		if ((y - r + i) >= 0 && (y - r + i) < HEIGHT)
		{
			int len = (int)sqrt((float)(r * r - (r - i) * (r - i))) * 2;
			int xofs = x - len / 2;

			// left border
			if (xofs < 0)
			{
				len += xofs;
				xofs = 0;
			}

			// right border
			if (xofs + len >= 680)
			{
				len -= (xofs + len) - 680;
			}
			int ofs = (y - r + i) * PITCH + xofs;
			int gofs = (y - r + i) * 680 + xofs;
			
			// note that len may be 0 at this point, 
			// and no pixels get drawn!
			for (j = 0; j < len; j++)
			{
				if (grainmap[gofs + j] < 0)
				{
					switch (type)
					{
					case 1:
						((unsigned int*)gScreen->pixels)[ofs + j + 120] = 
						blend_avg(c, ((unsigned int*)gScreen->pixels)[ofs + j + 120]);
						break;
					case 2:
						((unsigned int*)gScreen->pixels)[ofs + j + 120] = 
						blend_add(c, ((unsigned int*)gScreen->pixels)[ofs + j + 120]);
						break;
					default:
						((unsigned int*)gScreen->pixels)[ofs + j + 120] = c;
						break;
					}
				}
			}
		}
	}
}