/*
"Boom Boom Driller

  Copyright (C) 2010 Jari Komppa

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.

http://iki.fi/sol/
*/

#include "charge.h"
#include <float.h>


ReplayEvent replaybuffer[1024]; // 1024 bombs should be enough for everyone
int gReplayIndex;

CFL *cfl;

char * gCreditsText = "";

// Screen surface
SDL_Surface *gScreen;

// Level background
SDL_Surface *gBackground;

// Command strip
SDL_Surface *gCommands, *gCommandsHot, *gCommandsActive, *gCommandsDown;

// "check" layer
SDL_Surface *gCheckLayer = NULL;

// block mask
SDL_Surface *gBlockMask = NULL;

// Mouse cursors
SDL_Surface *gCursors;

// Bomb icons
SDL_Surface *gBombs;

// Drill icons
SDL_Surface *gDrills;


// Direction arrows
SDL_Surface *gDirection;

// Radials (radial menu)
SDL_Surface *gRadials;

// Font
SDL_Surface *gFont;

// Menu background
SDL_Surface *gMenubg;

// Splash screen
SDL_Surface *gSplash;

// big font
SDL_Surface *gBigFont;

// FFT texture
SDL_Surface *gFFTTexture;

SDL_Surface *gFlag = NULL;
SDL_Surface *gMap = NULL;

int gCurrentCampaign = 0;

MersenneTwister prand;
MersenneTwister vrand;

int gMusicVolume;
int gSfxVolume;
int gFmodTuneChannel;
char *gFmodTunePtr;

DrillTypes *drillinfo;
BombTypes *bombinfo;
GrainTypes *graininfo;
MouseRegion *mouseinfo;
Campaigns *campaigninfo;
int gCampaignCount;
int gGrainTypes;
int gReplicateParticle;

DelayedDig *gDelayedDigRoot = NULL;
Level *gCurrentLevel = NULL;
Level *gLevelList = NULL;

int gSmokeCap = 500;
int gExplosionCap = 200;
int gSparkleCap = 100;
int gFPSCap = 50;
int gEnableLittleTouches = 1;
int gPerfInfo = 0;
int gCampaignScore = 0;
int gTotalValuables;
int gCurrentValuables;
int gTotalNoharm;
int gCurrentNoharm;

int gGameState = STATE_SPLASH;
int gIngameState;

int gCheckActive = 0;
int gStateStartTick = 0;
int gTextStartTick = 0;
int gCurrentMoney;
int gCurrentDugTotal;
int grainmap[680*600];
Grain particle[680*600]; // max. number of particles
StateRepository *gStateRoot = NULL;

Level::~Level()
{
	SDL_FreeSurface(thumb);
	free((void*)desc);
	free((void*)levelfile);
	free((void*)bgfile);
	free((void*)skinfile);
	free((void*)name);
	free((void*)filename);
	free((void*)accuracyfile);
	free((void*)blockfile);
}

Level::Level()
{
	thumb = NULL;
	name = NULL;
	desc = NULL;
	levelfile = NULL;
	bgfile = NULL;
	skinfile = NULL;
	next = NULL;
	accuracyfile = NULL;
	blockfile = NULL;
}



void changestate(int newstate)
{
	if (newstate == STATE_SINGLE)
	{
		refresh_levels();
	}
	if (newstate == STATE_CAMPAIGN)
	{
		load_campaign(campaigninfo[gCurrentCampaign].filename);
	}
	gMouseActive = 0;
	gMouseDown = 0;
	gMouseOver = 0;
	gMouseState = MOUSE_IDLE;
	gGameState = newstate;
	gStateStartTick = SDL_GetTicks();
}

// Entry point
int main(int argc, char *argv[])
{
	_control87(_PC_24, MCW_PC);

	cfl = CFL::create("boom.cfl");
/*
    if (FSOUND_GetVersion() < FMOD_VERSION)
    {
        printf("Error : You are using the wrong FMOD DLL version!  You should be using FMOD %.02f\n", FMOD_VERSION);
        exit(3);
    }
    
    FSOUND_Init(44100, 16, 0);
*/
	// Initialize SDL's subsystems - in this case, only video.
    if (SDL_Init(SDL_INIT_VIDEO) < 0)
	{
        //fprintf(stderr, "Unable to init SDL: %s\n", SDL_GetError());
        exit(4);
    }

	// Register SDL_Quit to be called at exit; makes sure things are
	// cleaned up when we quit.
    atexit(SDL_Quit);
    
	// Attempt to create a WIDTHxHEIGHT window with 32bit pixels.
/*
	if (argc > 1)
	*/
		gScreen = SDL_SetVideoMode(WIDTH, HEIGHT, 32, SDL_SWSURFACE);
/*
	else
		gScreen = SDL_SetVideoMode(WIDTH, HEIGHT, 32, SDL_SWSURFACE | SDL_FULLSCREEN);
	*/
	
	// If we fail, return error.
    if (gScreen == NULL) 
	{
        //fprintf(stderr, "Unable to set up video: %s\n", SDL_GetError());
        exit(2);
    }

	init();

	// Main loop: loop forever.
    while (1)
    {
		int newframe;
		newframe = 1;

		switch (gGameState)
		{
		case STATE_SPLASH:
			splash_render();
			if (!splash_events())
				return 0;
			break;
		case STATE_MENU:
			menu_render();
			if (!menu_events())
				return 0;
			break;			
		case STATE_SINGLE:
			single_render();
			if (!menu_events())
				return 0;
			break;			
		case STATE_CREDITS:
			credits_render();
			if (!menu_events())
				return 0;
			break;			
		case STATE_SELECTCAMPAIGN:
			selectcampaign_render();
			if (!menu_events())
				return 0;
			break;			
		case STATE_CAMPAIGN:
			campaign_render();
			if (!menu_events())
				return 0;
			break;			
		case STATE_OPTIONS:
			options_render();
			if (!menu_events())
				return 0;
			break;			
		case STATE_CONFIRM:
			confirm_render();
			if (!menu_events())
				return 0;
			break;
		case STATE_INGAME:
			newframe = ingame_render();
			if (!ingame_events())
				return 0;
			break;
		}
		if (newframe)
		{
			SDL_UpdateRect(gScreen, 0, 0, WIDTH, HEIGHT);    
		}
    }
    return 0;
}
