#include "charge.h"

void add_smoke(int x, int y);

#define EXPLOSION_TTL 50

int gActiveSparkles = 0;
int gActiveExplosions = 0;
int gActiveSmokes = 0;

struct Sparkle
{
	Sparkle() { gActiveSparkles++; }
	~Sparkle() { gActiveSparkles--; }
	float x, y, xi, yi;
	int ttl;	
	Sparkle *next;
};

struct Explosion
{
	Explosion() { gActiveExplosions++; }
	~Explosion() { gActiveExplosions--; }
	int x, y;
	int r;
	int ttl;
	int tta;
	int iterations;
	int children;
	Explosion *next;
};

struct Smoke
{
	Smoke() { gActiveSmokes++; }
	~Smoke() { gActiveSmokes--; }
	float x, y;
	float yi;
	int ttl;
	int color;
	Smoke *next;
};

Smoke *smokelist = NULL;
Sparkle *sparklelist =  NULL;
Explosion *explosionlist = NULL;


void particle_physics()
{
	Sparkle *walker = sparklelist;
		
	while (walker)
	{
		walker->ttl--;

		walker->x += walker->xi;
		walker->y += walker->yi;

		walker->xi *= 0.9f;
		walker->yi *= 0.9f;

		if (walker->x >= 675 ||
			walker->x < 5 ||
			walker->y < 5 ||
			walker->y >= 595)
			walker->ttl = -1;

		walker = walker->next;
	}

	// sweep sparkles

	while (sparklelist && sparklelist->ttl <= 0)
	{
		Sparkle *d = sparklelist;
		sparklelist = sparklelist->next;
		delete d;
	}
	walker = sparklelist;
	Sparkle *last = sparklelist;
	while (walker)
	{
		if (walker->ttl <= 0)
		{
			Sparkle * d = walker;
			last->next = walker->next;
			walker = walker->next;
			delete d;
		}
		else
		{
			last = walker;
			walker = walker->next;
		}
	}

	Explosion * ew = explosionlist;
	while (ew)
	{
		ew->tta --;
		if (ew->tta <= 0 && ew->iterations)
		{
			int i;
			ew->iterations--;
			for (i = 0; i < ew->children; i++)
			{
				int j, k;
				k = 0;
				j = 0;
				do 
				{
					float dir = (float)(vrand.genrand_real1() * 2 * PI);
					float dist = (float)(((vrand.genrand_real1() / 2.0) + 0.5) * ew->r);				
					k = add_explosion(ew->x + (int)(sin(dir)*dist),
								ew->y + (int)(cos(dir)*dist), 
								ew->r, 
								ew->iterations, 
								ew->children);
					j++;
				}
				while (k == 0 && j < 5);
			}
		}
		ew->ttl--;

		ew = ew->next;
	}

	while (explosionlist && explosionlist->ttl <= 0)
	{
		Explosion *d = explosionlist;
		explosionlist = explosionlist->next;
		delete d;
	}
	ew = explosionlist;
	Explosion *le = explosionlist;
	while (ew)
	{
		if (ew->ttl <= 0)
		{
			Explosion * d = ew;
			le->next = ew->next;
			ew = ew->next;
			delete d;
		}
		else
		{
			le = ew;
			ew = ew->next;
		}
	}

	Smoke * sw = smokelist;
	while (sw)
	{
		sw->ttl--;

		sw->y += sw->yi;
		//sw->yi = -0.75f;
		int p = (int)sw->x + (int)(sw->y) * 680;
		if (p < 0) 
		{
			sw->ttl = -1;
		}
		else
		if (grainmap[p] >= 0)
		{
			if (grainmap[p + 1] < 0)
			{
				sw->x++;
			}
			else
			{
				if (grainmap[p - 1] < 0)
				{
					sw->x--;
				}
				else
				{
					if (grainmap[p + 2] < 0)
					{
						sw->x += 2;
					}
					else
					{
						if (grainmap[p - 2] < 0)
						{
							sw->x -= 2;
						}
						else
						{
							sw->y -= sw->yi;
						}
					}
				}
			}
		}
		p = (int)sw->x + (int)(sw->y) * 680;
		if (p < 0 || grainmap[p] >= 0)
			sw->ttl = -1;

		sw = sw->next;
	}

	while (smokelist && smokelist->ttl <= 0)
	{
		Smoke *d = smokelist;
		smokelist = smokelist->next;
		delete d;
	}
	sw = smokelist;
	Smoke *ls = smokelist;
	while (sw)
	{
		if (sw->ttl <= 0)
		{
			Smoke * d = sw;
			ls->next = sw->next;
			sw = sw->next;
			delete d;
		}
		else
		{
			ls = sw;
			sw = sw->next;
		}
	}

}

void bloom()
{
	Explosion * ew = explosionlist;
	while (ew)
	{
		float size = ew->ttl / (float)EXPLOSION_TTL;
		size = SMOOTHSTEP(size);
		int color = (int)(0x0f * size) << 16 |
					(int)(0x0f * size * size * size) << 8 |
					(int)(0x07 * size * size * size * size * size);
		drawcircle((int)ew->x + 120, (int)ew->y, (int)(ew->r * size), color, 2);
		ew = ew->next;
	}
}

void render_particles()
{
	Smoke * sw = smokelist;
	while (sw)
	{
		float size = sw->ttl / (float)200;
		size = SMOOTHSTEP(size);
		drawcirclez((int)sw->x, (int)sw->y, (int)(16 * size), sw->color, 1);
		sw = sw->next;
	}

	Explosion * ew = explosionlist;
	while (ew)
	{
		float size = ew->ttl / (float)EXPLOSION_TTL;
		size = SMOOTHSTEP(size);
		int color = (int)(0xff * size) << 16 |
					(int)(0xff * size * size * size) << 8 |
					(int)(0x3f * size * size * size * size * size);
		drawcirclez((int)ew->x, (int)ew->y, (int)(ew->r * size), color, 0);
		ew = ew->next;
	}

	Sparkle * walker = sparklelist;
	while (walker)
	{
		walker->x += walker->xi;
		walker->y += walker->yi;
		
		int x = (int)walker->x;
		int y = (int)walker->y;

		((unsigned int*)gScreen->pixels)[120 + (y) * gScreen->pitch / 4 + x] = 0xffffff;
		((unsigned int*)gScreen->pixels)[120 + (y+1) * gScreen->pitch / 4 + x+1] = 0xffffff;
		((unsigned int*)gScreen->pixels)[120 + (y+1) * gScreen->pitch / 4 + x-1] = 0xffffff;
		((unsigned int*)gScreen->pixels)[120 + (y+1) * gScreen->pitch / 4 + x] = 0xffffff;
		((unsigned int*)gScreen->pixels)[120 + (y+2) * gScreen->pitch / 4 + x] = 0xffffff;

		walker = walker->next;
	}
}

void reset_particles()
{
	Sparkle * walker = sparklelist;
	while (walker)
	{
		Sparkle * d = walker;
		walker = walker->next;
		delete d;
	}
	sparklelist = NULL;

	Explosion *ew = explosionlist;
	while (ew)
	{
		Explosion *d = ew;
		ew = ew->next;
		delete d;
	}
	explosionlist = NULL;

	Smoke *sw = smokelist;
	while (sw)
	{
		Smoke *d = sw;
		sw = sw->next;
		delete d;
	}
	smokelist = NULL;
}

void add_sparkle(int x, int y)
{
	if (!gEnableLittleTouches)
		return;
	if (gActiveSparkles > gSparkleCap)
		return;
	if (x < 5 || x > 675 || y < 5 || y > 595 || (vrand.genrand_int32() & 15) != 0)
		return;
	Sparkle *s = new Sparkle;
	s->next = sparklelist;
	sparklelist = s;
	s->x = (float)x;
	s->y = (float)y;
	s->ttl = 10 + vrand.genrand_int32()&15;
	float dir = (float)(vrand.genrand_real1() * 2 * PI);
	s->xi = sin(dir) * 2.0f;
	s->yi = cos(dir) * 2.0f;
}

int add_explosion(int x, int y, int radius, int iterations, int children)
{
	if (x < 0 || x > 679 || y < 0 || y > 599 || grainmap[x + y * 680] >= 0)
		return 0;
	if (gActiveExplosions >= gExplosionCap) 
		return 1;
	add_smoke(x, y);

	Explosion *e = new Explosion();
	e->iterations = iterations;
	e->x = x;
	e->y = y;
	e->r = radius;
	e->ttl = (int)(EXPLOSION_TTL * ((vrand.genrand_real1()/2.0f) + 0.5f));
	e->tta = 5;
	e->children = children;
	e->next = explosionlist;
	explosionlist = e;
	return 1;
}

void add_smoke(int x, int y)
{
	if (gActiveSmokes >= gSmokeCap) 
		return;
	Smoke *s = new Smoke;
	s->next = smokelist;
	smokelist = s;
	s->x = (float)x;
	s->y = (float)y;
	s->yi = (float)(-(vrand.genrand_real1() * 0.75 + 0.5));
	s->ttl = 150 + vrand.genrand_int32() % 200;
	s->color = ((vrand.genrand_int32()&0x1f) + 0x33) * 0x010101;
}

